#! /bin/bash


print_help()
{
    cat << EOF
usage:  glustershd_flush.sh [-h] [-v] <volume name> [flush]

-h : View this help
-v : Turn on verbose

For a replication glusterfs volume, data is replicated over
multiple glusterfs bricks.  This script detects if there is
data on bricks (on host that executes this script) have data
that is not replicated yet to other bricks due to various
reasons (e.g. network is down), for example

    $ glustershd_flush.sh glustersharevol
    local clean

If gluserfs_flush.sh is run with "flush" option, the glustershd_flush.sh
will get blocked until all the data is replicated to other bricks.

    $ glustershd_flush.sh glustersharevol flush
    Flush done!

Exit status:
 0 if flush is done successfully or local is not dirty
 1 if local is dirty
 2 if there is error and the error message will be printed out
EOF
}

check_local_dirty()
{
    touch /var/lib/glusterd/glustershd/flushing
    [ "$verbose" == "1" ] && echo "Trigger healing process"
    msg=`gluster volume heal $VOLUME_NAME 2>&1`
    res=$?
    [ "$verbose" == "1" ] && echo "Healing process done"
    if [ $res -ne 0 ]; then
        rm -f /var/lib/glusterd/glustershd/flushing > /dev/null 2>&1
        return 100
    fi
    while [ -e /var/lib/glusterd/glustershd/flushing ]; do
        [ "$verbose" == "1" ] && echo "sleep looping until flushing is done"
        sleep 2
    done

    local_dirty_files=`ls /var/lib/glusterd/glustershd/local_dirty*`
    res=0
    for f in ${local_dirty_files}; do
        val=`cat $f`
        res=$(( $res + $val ))
    done
    msg="$res"
    [ "${res}" == "0" ] && return 0 || return 1
}

flush_local_dirty()
{
    while [ 1 ]; do
        check_local_dirty
        res=$?
        if [ "$res" == "1" ]; then
            [ "$verbose" == "1" ] && echo "local still dirty ($msg), sleep and recheck later"
            sleep 3
        elif [ "$res" == "100" ]; then
            if [ -n "`echo $msg | grep 'Self-heal daemon is not running'`" ]; then
                [ "$verbose" == "1" ] && echo "Self-heal daemon is not running, sleep and recheck later"
                sleep 3
            else
                echo "ERROR: $msg"
                return 2
            fi
        else
            return 0
        fi
    done
}


msg=""
verbose=0

while getopts "?vh" opt; do
    case $opt in
    v)
        verbose=1
        ;;
    h)
        print_help
        exit 0
        ;;
    ?)
        print_help
        exit 0
        ;;
    esac
done

# Move aside the optional arguments
shift $(( $OPTIND - 1 ))

VOLUME_NAME=$1
cmd=$2
ret_code=0

if [ -z "$VOLUME_NAME" ]; then
    print_help
    exit 2
fi

touch /var/lib/glusterd/glustershd/check_local_dirty
touch /var/lib/glusterd/glustershd/flush_nonblock
if [ "$cmd" == "flush" ]; then
    flush_local_dirty
    ret_code=$?
    [ "$ret_code" == "0" ] && echo "Flush done!"
else
    check_local_dirty
    res=$?
    if [ "$res" == "0" ]; then
        echo "local clean"
        ret_code=0
    elif [ "$res" == "1" ]; then
        echo "local dirty ($msg)"
        ret_code=1
    else
        echo "ERROR: $msg"
        ret_code=2
    fi
fi
rm /var/lib/glusterd/glustershd/flush_nonblock
rm /var/lib/glusterd/glustershd/check_local_dirty
exit $ret_code
